$(document).ready(function () {

    var queryHandlers = {};

    var SERVICE_QUERY_TIMEOUT_MS = 10000;
    var SERVICE_QUERY_INTERVAL_MS = 1000;

    var queryHandlersWatchdogHandler = function () {
        Object.keys(queryHandlers).forEach(function (key) {
            var h = queryHandlers[key];
            var elapsed = Date.now() - h.timestamp;
            if (elapsed > SERVICE_QUERY_TIMEOUT_MS) {
                delete queryHandlers[key];
                h.failure({status: 400, reason: 'REQUEST_TIMEOUT', info: 'Timeout query exceeded'});
            }
        });
        setTimeout(queryHandlersWatchdogHandler, SERVICE_QUERY_INTERVAL_MS);
    };
    setTimeout(queryHandlersWatchdogHandler, SERVICE_QUERY_INTERVAL_MS);

    function sendWsRequest(client, config) {
        var query = "NO DATA";
        var id = generateUUID();
        var data = JSON.stringify({requestId: id, realm: config.url, payload: config.data});
        if (config.data) {
            query = syntaxHighlight(data, null, null);
            $("#requestOut").html(query);
        }

        return new Promise(function (resolve, reject) {
            queryHandlers[id] = {success: resolve, failure: reject, timestamp: Date.now()};
            client.send("/app" + config.url, {}, data);
        });
    }

    $('#btnClk').click(function () {
        var id = $("#metricQueryId").val();
        sendWsRequest(client, {
            url: "/api/metric/list",
            data: {id: id}
        }).then(function (e) {
            $("#statusOut").text("Found " + e.length + " metrics.");
        }).catch(function (e) {
            $("#statusOut").text("Error " + e.reason + ": " + e.info);
        });
    });

    $('#btnProfiles').click(function () {
        var id = $("#profilesQueryId").val();

        sendWsRequest(client, {
            url: "/api/profile/list",
            data: {id: id}
        }).then(function (e, h) {
            $("#statusOut").text("Found " + e.length + " profiles.");
        }).catch(function (e) {
            $("#statusOut").text("Error " + e.reason + ": " + e.info);
        });
    });

    $('#btnNodeList').click(function () {
        var id = $("#nodeId").val();

        sendWsRequest(client, {
            url: "/api/node/list",
            data: {id: id}
        }).then(function (e) {
            $("#statusOut").text("Found " + e.length + " nodes.");
        }).catch(function (e) {
            $("#statusOut").text("Error " + e.reason + ": " + e.info);
        });
    });

    $('#btnNodeEdit').click(function () {
        var id = $("#editNodeId").val();
        var profiles = $("#editNodeProfiles").val();
        var profileList = profiles.split(",");
        sendWsRequest(client, {
            url: "/api/node/update",
            data: {id: id, profiles: profileList}
        }).then(function (e) {
            $("#statusOut").text("Node saved.");
        }).catch(function (e) {
            $("#statusOut").text("Error " + e.reason + ": " + e.info);
        });
    });

    $('#btnNodeStatus').click(function () {
        var id = $("#statusNodeId").val();

        sendWsRequest(client, {
            url: "/api/node_status/list",
            data: {id: id}
        }).then(function (e) {
            $("#statusOut").text("Found " + e.length + " nodes status.");
        }).catch(function (e) {
            $("#statusOut").text("Error " + e.reason + ": " + e.info);
        });
    });

    $('#btnNodeCacheList').click(function () {
        var id = $("#nodeCacheId").val();
        var mediaId = $("#nodeCacheMediaId").val();
        var closed = $("#nodeCacheClosed").prop("checked");
        var nodeCacheNodeIp = $("#nodeCacheNodeIp").val();
        var nodeCacheNodeHostName = $("#nodeCacheNodeHostName").val();
        var nodeCacheStatus = $("#nodeCacheStatus").val();
        var nodeCacheName = $("#nodeCacheName").val();

        sendWsRequest(client, {
            url: "/api/stream/list",
            data: {
                node: id,
                mediaId: mediaId,
                name: nodeCacheName,
                status: nodeCacheStatus,
                showClosed: closed,
                nodeIp: nodeCacheNodeIp,
                nodeHostName: nodeCacheNodeHostName
            }
        }).then(function (e) {
            $("#statusOut").text("Found " + e.length + " streams.");
        }).catch(function (e) {
            $("#statusOut").text("Error " + e.reason + ": " + e.info);
        });
    });

    $('#btnNodeHistoryList').click(function () {
        var id = $("#nodeHistoryId").val();
        var mediaId = $("#nodeHistoryMediaId").val();
        var nodeHistoryNodeHostName = $("#nodeHistoryNodeHostName").val();
        var nodeHistoryNodeIp = $("#nodeHistoryNodeIp").val();
        var nodeHistoryName = $("#nodeHistoryName").val();
        var nodeHistoryPublisherId = $("#nodeHistoryPublisherId").val();
        var nodeHistoryStatus = $("#nodeHistoryStatus").val();

        sendWsRequest(client, {
            url: "/api/stream/history",
            data: {
                node: id,
                mediaId: mediaId,
                name: nodeHistoryName,
                publisher: nodeHistoryPublisherId,
                status: nodeHistoryStatus,
                nodeIp: nodeHistoryNodeIp,
                nodeHostName: nodeHistoryNodeHostName,
                limit: 30,
                offset: 0
            }
        }).then(function (e) {
            $("#statusOut").text("Found " + e.length + " streams.");
        }).catch(function (e) {
            $("#statusOut").text("Error " + e.reason + ": " + e.info);
        });
    });

    $('#btnNodeMetricList').click(function () {
        var id = $("#nodeMetricNodeId").val();
        var mediaId = $("#nodeMetricMediaId").val();
        sendWsRequest(client, {
            url: "/api/stream/metrics",
            data: {node: id, mediaId: mediaId, limit: 10, offset: 0}
        }).then(function (e) {
            $("#statusOut").text("Found " + e.length + " metric packages.");
        }).catch(function (e) {
            $("#statusOut").text("Error " + e.reason + ": " + e.info);
        });
    });

    $('#btnDeleteProfile').click(function () {
        var id = $("#deleteProfileId").val();
        sendWsRequest(client, {
            url: "/api/profile/delete",
            data: {id: id}
        }).then(function (e) {
            $("#statusOut").text("Profile with id " + id + " deleted.");
        }).catch(function (e) {
            $("#statusOut").text("Error " + e.reason + ": " + e.info);
        });
    });

    $('#btnNodeDelete').click(function () {
        var id = $("#deleteNodeId").val();
        sendWsRequest(client, {
            url: "/api/node/delete",
            data: {id: id}
        }).then(function (e) {
            $("#statusOut").text("Node with id " + id + " deleted.");
        }).catch(function (e) {
            $("#statusOut").text("Error " + e.reason + ": " + e.info);
        });
    });

    $('#btnRuleDelete').click(function () {
        var id = $("#deleteRuleId").val();
        sendWsRequest(client, {
            url: "/api/rule/delete",
            data: {id: id}
        }).then(function (e) {
            $("#statusOut").text("Rule with id " + id + " deleted.");
        }).catch(function (e) {
            $("#statusOut").text("Error " + e.reason + ": " + e.info);
        });
    });

    $('#btnAlarmDelete').click(function () {
        var id = $("#deleteAlarmId").val();
        sendWsRequest(client, {
            url: "/api/alarm/delete",
            data: {id: id}
        }).then(function (e) {
            $("#statusOut").text("Alarm with id " + id + " deleted.");
        }).catch(function (e) {
            $("#statusOut").text("Error " + e.reason + ": " + e.info);
        });
    });

    $('#btnAlarmEdit').click(function () {
        var id = $("#editAlarmId").val();
        var typeId = $("#editAlarmTypeId").val();
        var name = $("#editAlarmName").val();
        var value = $("#editAlarmValue").val();
        var metric = $("#editAlarmMetric").val();
        var node = $("#editAlarmNode").val();
        var time = $("#editAlarmTime").val();
        sendWsRequest(client, {
            url: "/api/alarm/update",
            data: {
                id: id,
                type: typeId,
                name: name,
                value: value,
                metric: metric,
                node: node,
                time: time
            }
        }).then(function (e) {
            $("#statusOut").text("Alarm with id " + id + " updated.");
        }).catch(function (e) {
            $("#statusOut").text("Error " + e.reason + ": " + e.info);
        });
    });

    $('#btnAlarmNew').click(function () {
        var id = $("#newAlarmId").val();
        var typeId = $("#newAlarmTypeId").val();
        var name = $("#newAlarmName").val();
        var value = $("#newAlarmValue").val();
        var metric = $("#newAlarmMetric").val();
        var node = $("#newAlarmNode").val();
        var time = $("#newAlarmTime").val();
        sendWsRequest(client, {
            url: "/api/alarm/create",
            data: {
                id: id,
                type: typeId,
                name: name,
                value: value,
                metric: metric,
                node: node,
                time: time
            }
        }).then(function (e) {
            $("#statusOut").text("Alarm with name " + name + " created.");
        }).catch(function (e) {
            $("#statusOut").text("Error " + e.reason + ": " + e.info);
        });
    });

    $('#btnRuleEdit').click(function () {
        var id = $("#editRuleId").val();
        var typeId = $("#editRuleTypeId").val();
        var name = $("#editRuleName").val();
        var value = $("#editRuleValue").val();
        var profiles = $("#editRuleProfiles").val();
        var profileList = profiles.split(",");
        sendWsRequest(client, {
            url: "/api/rule/update",
            data: {
                id: id,
                type: typeId,
                name: name,
                value: value,
                profiles: profileList
            }
        }).then(function (e) {
            $("#statusOut").text("Rule with id " + id + " updated.");
        }).catch(function (e) {
            $("#statusOut").text("Error " + e.reason + ": " + e.info);
        });
    });

    $('#btnRuleNew').click(function () {
        var typeId = $("#newRuleTypeId").val();
        var name = $("#newRuleName").val();
        var value = $("#newRuleValue").val();
        var profiles = $("#newRuleProfiles").val();
        var profileList = [];
        profiles.split(",").forEach(function (e) {
            profileList.push(e);
        });

        sendWsRequest(client, {
            url: "/api/rule/create",
            data: {
                type: typeId,
                name: name,
                value: value,
                profiles: profileList
            }
        }).then(function (e) {
            $("#statusOut").text("Rule with name " + name + " created.");
        }).catch(function (e) {
            $("#statusOut").text("Error " + e.reason + ": " + e.info);
        });
    });

    $('#btnProfileNew').click(function () {
        var metricList = $("#newProfileMetrics").val();
        var ruleList = $("#newProfileRules").val();
        var name = $("#newProfileName").val();
        var rate = $("#newProfileRate").val();

        var metrics = [];
        metricList.split(",").forEach(function (e) {
            metrics.push(e);
        });
        var rules = [];
        ruleList.split(",").forEach(function (e) {
            rules.push(e);
        });

        sendWsRequest(client, {
            url: "/api/profile/create",
            data: {
                name: name,
                rate: rate,
                metrics: metrics,
                rules: rules
            }
        }).then(function (e) {
            $("#statusOut").text("Profile with name " + name + " created.");
        }).catch(function (e) {
            $("#statusOut").text("Error " + e.reason + ": " + e.info);
        });
    });


    $('#btnRuleLoad').click(function () {
        var id = $("#editRuleId").val();
        sendWsRequest(client, {
            url: "/api/rule/list",
            data: {id: id, profileId: ''}
        }).then(function (e) {
            if (e.length > 0) {
                var item = e[0];
                $("#editRuleName").val(item.name);
                $("#editRuleValue").val(item.value);
                $("#editRuleTypeId").val(item.type);
                $("#editRuleProfiles").val(item.profiles);
            }
            $("#statusOut").text("Load Rule with id " + id);
        }).catch(function (e) {
            $("#statusOut").text("Error " + e.reason + ": " + e.info);
        });

    });

    $('#btnProfileLoad').click(function () {
        var id = $("#editProfileId").val();
        sendWsRequest(client, {
            url: "/api/profile/list",
            data: {id: id}
        }).then(function (e) {
            if (e.length > 0) {
                var item = e[0];
                $("#editProfileRate").val(item.rate);
                $("#editProfileName").val(item.name);
                $("#editProfileRules").val(item.rules);
                $("#editProfileMetrics").val(item.metrics);
            }
            $("#statusOut").text("Load Profile with id " + id);
        }).catch(function (e) {
            $("#statusOut").text("Error " + e.reason + ": " + e.info);
        });
    });


    $('#btnNodeLoad').click(function () {
        var id = $("#editNodeId").val();
        sendWsRequest(client, {
            url: "/api/node/list",
            data: {id: id}
        }).then(function (e) {
            if (e.length > 0) {
                var item = e[0];
                $("#editNodeIp").val(item.host);
                $("#editNodePort").val(item.port);
                $("#editNodeProfiles").val(item.profiles);
            }
            $("#statusOut").text("Load Node with id " + id);
        }).catch(function (e) {
            $("#statusOut").text("Error " + e.reason + ": " + e.info);
        });
    });

    $('#btnAlarmLoad').click(function () {
        var id = $("#editAlarmId").val();
        sendWsRequest(client, {
            url: "/api/alarm/list",
            data: {id: id}
        }).then(function (e) {
            if (e.length > 0) {
                var item = e[0];
                $("#editAlarmName").val(item.name);
                $("#editAlarmTime").val(item.time);
                $("#editAlarmValue").val(item.value);
                $("#editAlarmMetric").val(item.metric);
                $("#editAlarmTypeId").val(item.type);
                $("#editAlarmNode").val(item.node);
            }
            $("#statusOut").text("Load Alarm with id " + id);
        }).catch(function (e) {
            $("#statusOut").text("Error " + e.reason + ": " + e.info);
        });
    });

    $('#btnProfileEdit').click(function () {
        var id = $("#editProfileId").val();
        var metricList = $("#editProfileMetrics").val();
        var ruleList = $("#editProfileRules").val();
        var name = $("#editProfileName").val();
        var rate = $("#editProfileRate").val();

        var metrics = [];
        metricList.split(",").forEach(function (e) {
            metrics.push(e);
        });
        var rules = [];
        ruleList.split(",").forEach(function (e) {
            rules.push(e);
        });

        sendWsRequest(client, {
            url: "/api/profile/update",
            data: {
                id: id,
                name: name,
                rate: rate,
                metrics: metrics,
                rules: rules
            }
        }).then(function (e) {
            $("#statusOut").text("Profile with id " + id + " updated.");
        }).catch(function (e) {
            $("#statusOut").text("Error " + e.reason + ": " + e.info);
        });
    });

    $('#btnAlarmList').click(function () {
        var id = $("#alarmId").val();

        sendWsRequest(client, {
            url: "/api/alarm/list",
            data: {id: id}
        }).then(function (e) {
            $("#statusOut").text("Load " + e.length + " Alarms");
        }).catch(function (e) {
            $("#statusOut").text("Error " + e.reason + ": " + e.info);
        });
    });

    $('#btnRuleList').click(function () {
        var ruleMediaId = $("#ruleId").val();
        var ruleProfile = $("#ruleProfileId").val();

        sendWsRequest(client, {
            url: "/api/rule/list",
            data: {id: ruleMediaId, profile: ruleProfile}
        }).then(function (e) {
            $("#statusOut").text("Load " + e.length + " Rules");
        }).catch(function (e) {
            $("#statusOut").text("Error " + e.reason + ": " + e.info);
        });
    });


    var wsType = window.location.protocol.replace(/:/g, '') == "https" ? "wss" : "ws";
    var client = Stomp.client(wsType + "://" + window.location.hostname + ":" + window.location.port + "/ws");
    client.connect({}, function () {
        client.subscribe("/alarms", function (msg) {
            msg.body = JSON.parse(msg.body);
            var el = $("<div>");
            el.html(syntaxHighlight(JSON.stringify(msg.body, null, 4)));
            $("#alarmPanel").append(el)
        });
        client.subscribe("/user/service", function (msg) {
            msg.body = JSON.parse(msg.body);
            $("#responseOut").html(syntaxHighlight(JSON.stringify(msg.body, null, 4)));

            if (queryHandlers[msg.body.requestId]) {
                var handler = queryHandlers[msg.body.requestId];
                delete queryHandlers[msg.body.requestId];
                if (msg.body.status >= 200 && msg.body.status <= 299) {
                    handler.success(msg.body.payload);
                } else {
                    handler.failure(msg.body);
                }
            } else {
                alert("Unknown response " + JSON.stringify(msg.body));
            }
        });
    });

    var subscriptions = {};

    var closeSubscription = function (config) {
        id = config.client.id;
        config.client.unsubscribe();
        $("#" + id).remove();
        delete subscriptions[id];
    }

    var addToSubscriptionList = function (config) {
        subscriptions[config.id] = config;
        var row = "<tr id='" + config.client.id + "'><td>" + config.client.id + "</td><td>" + config.mediaId + "</td><td>" + config.nodeId + "</td><td><button id='button_" + config.client.id + "'>Close</button></td></tr>";
        $("#subscriptionList").append(row);
        $("#button_" + config.client.id).click(function () {
            closeSubscription(config);
        });
    }

    $('#btnWebSocket').click(function () {
        var mediaId = $("#stompMediaId").val();
        var nodeId = $("#stompNodeId").val();
        var display = window.open("chart-display.html");
        var c = client.subscribe("/stream/" + nodeId + "/" + mediaId, function (msg) {
            msg.body = JSON.parse(msg.body);
            $("#messageOut").html(syntaxHighlight(JSON.stringify(msg, null, 4)));
            if (display.hasOwnProperty("addData")) {
                display['addData'](msg.body);
            }
        });
        var config = {client: c, mediaId: mediaId, nodeId: nodeId, display: display};
        addToSubscriptionList(config);

        display.addEventListener("load", function () {
            display["init"](mediaId, metrics);
        });
    });
});