function syntaxHighlight(json) {
    if (typeof json != 'string') {
        json = JSON.stringify(json, undefined, 2);
    }
    json = json.replace(/&/g, '&amp;').replace(/</g, '&lt;').replace(/>/g, '&gt;');
    return json.replace(/("(\\u[a-zA-Z0-9]{4}|\\[^u]|[^\\"])*"(\s*:)?|\b(true|false|null)\b|-?\d+(?:\.\d*)?(?:[eE][+\-]?\d+)?)/g, function (match) {
        var cls = 'number';
        if (/^"/.test(match)) {
            if (/:$/.test(match)) {
                cls = 'key';
            } else {
                cls = 'string';
            }
        } else if (/true|false/.test(match)) {
            cls = 'boolean';
        } else if (/null/.test(match)) {
            cls = 'null';
        }
        return '<span class="' + cls + '">' + match + '</span>';
    });
}

function generateUUID() { // Public Domain/MIT
    var d = new Date().getTime();
    if (typeof performance !== 'undefined' && typeof performance.now === 'function'){
        d += performance.now(); //use high-precision timer if available
    }
    return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function (c) {
        var r = (d + Math.random() * 16) % 16 | 0;
        d = Math.floor(d / 16);
        return (c === 'x' ? r : (r & 0x3 | 0x8)).toString(16);
    });
}

function sendRequest(config) {
    var query = "NO DATA";
    if (config.data) {
        query = syntaxHighlight(JSON.stringify(JSON.parse(config.data), null, null));
    }
    $("#requestOut").html(config.type + ": " + config.url + " \"" + config.contentType + "\"<br />" + query);
    config.error = function (req, status, err) {
        var resp = JSON.parse(req.responseText);
        alert("Error: " + resp.reason + " Info " + resp.info);
    }
    config.success = function (data) {
        if ( config.filler && data.payload.length > 0 ) {
            config.filler(data.payload[0]);
        }
        $("#responseOut").html(syntaxHighlight(JSON.stringify(data, null, 4)));
    }
    $.ajax(config);
}

function makeChart(containerId, title) {
    var chart = new CanvasJS.Chart(containerId, {
        title: {
            text: title
        },
        axisY: {
            includeZero: false
        },
        axisX: {
            valueFormatString: "H:mm:ss",
            labelAngle: -50
        },
        data: [{
            type: "line",
            dataPoints: []
        }]
    });
    return chart;
}

var metrics = [
    {
        "id": 4,
        "name": "Video rate",
        "note": "",
        "enumName": "VIDEO_RATE"
    },
    {
        "id": 5,
        "name": "Video sync",
        "note": "",
        "enumName": "VIDEO_SYNC"
    },
    {
        "id": 6,
        "name": "Video FPS",
        "note": "",
        "enumName": "VIDEO_FPS"
    }
];